library(Seurat)
library(SingleR)
library(ggplot2)
library(cowplot)
library(gridExtra)
library(RColorBrewer)
library(viridis)
library(scales)
library(ComplexHeatmap)

options(bitmapType="cairo")

setwd("../")

load("./analysis/sc10x.id.rda")
load("./analysis/singler_objects.RData")

if (!dir.exists(paste0("./analysis/vis/diy"))){
  dir.create(paste0("./analysis/vis/diy"))
}

Idents(sc10x) <- "samples"
sc10x <- subset(sc10x,idents=c("D17PrPzF_Via","D27PrPzF_Via","D35PrPzF_Via","BPH327PrGF_Via","BPH340PrGF_Via","BPH342PrF_Via"))

sc10x$Phenotype <- sc10x$samples
Idents(sc10x) <- "Phenotype"
sc10x <- RenameIdents(sc10x,"D17PrPzF_Via"="Normal Pz")
#sc10x <- RenameIdents(sc10x,"D17PrTzF_Via"="Normal")
sc10x <- RenameIdents(sc10x,"D27PrPzF_Via"="Normal Pz")
#sc10x <- RenameIdents(sc10x,"D27PrTzF_Via"="Normal")
sc10x <- RenameIdents(sc10x,"D35PrPzF_Via"="Normal Pz")
#sc10x <- RenameIdents(sc10x,"D35PrTzF_Via"="Normal")
sc10x <- RenameIdents(sc10x,"BPH327PrGF_Via"="BPH")
sc10x <- RenameIdents(sc10x,"BPH340PrGF_Via"="BPH")
sc10x <- RenameIdents(sc10x,"BPH342PrF_Via"="BPH")
sc10x$Phenotype <- Idents(sc10x)
sc10x$Phenotype <- factor(sc10x$Phenotype,levels=c("Normal Pz","BPH"))

sc10x$`Cell.Lineage` <- sc10x$lin
Idents(sc10x) <- "Cell.Lineage"
sc10x$`Cell.Lineage` <- Idents(sc10x)
sc10x$`Cell.Lineage` <- factor(sc10x$`Cell.Lineage`,levels=c("Epi","FMSt","Endo","Leu"))
Idents(sc10x) <- "Cell.Lineage"

postscript(paste0("./analysis/vis/diy/UMAP_lin.eps"))
DimPlot(sc10x)+scale_color_viridis(discrete=TRUE,option="magma")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_lin.split.eps"))
DimPlot(sc10x,split.by="Phenotype")+scale_color_viridis(discrete=TRUE,option="magma")+theme_cowplot()
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_lin.anchor.eps"))
DotPlot(sc10x,features=rev(c("EPCAM","CDH1","DCN","MYL9","PECAM1","VWF","PTPRC")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

Idents(sc10x) <- "lin"
leu <- WhichCells(sc10x,idents=c("Epi","FMSt","Endo"),invert=TRUE)
sc10x$`Cell.Type` <- sc10x$pops
Idents(sc10x) <- "Cell.Type"
sc10x <- SetIdent(sc10x,cells=leu,value="Leu")
sc10x$`Cell.Type` <- Idents(sc10x)
sc10x$`Cell.Type` <- factor(sc10x$`Cell.Type`,levels=c("BE","LE","Hillock","Club","NE","Fib","SM","Endo","Leu"))
Idents(sc10x) <- "Cell.Type"

postscript(paste0("./analysis/vis/diy/UMAP_pops.eps"))
DimPlot(sc10x)+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_pops.split.eps"))
DimPlot(sc10x,split.by="Phenotype")+theme_cowplot()
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_pops.anchor.eps"))
DotPlot(sc10x,features=rev(c("KRT5","KLK3","KRT13","SCGB3A1","CHGA","DCN","MYL9","PECAM1","PTPRC")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

Idents(sc10x) <- "integrated_snn_res.5"
postscript(paste0("./analysis/vis/diy/UMAP_lin.res5.eps"))
DimPlot(sc10x)+theme_cowplot()
dev.off()

lin.se <- HumanPrimaryCellAtlasData()
singler.lin@listData$scores <- singler.lin@listData$scores[,c(
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Epithelial_cells","Keratinocytes","Neuroepithelial_cell"),
    c("Epithelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Smooth_muscle_cells","Fibroblasts","Chondrocytes","Osteoblasts","MSC","Tissue_stem_cells"),
    c("Stromal cells","Fibroblasts")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Endothelial_cells","Erythroblast","Platelets"),
    c("Endothelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("DC","B_cell","Neutrophil","T_cells","Monocyte","Macrophage","NK_cell","Neutrophils","CMP","GMP","MEP","Myelocyte","Pre-B_cell_CD34-","Pro-B_cell_CD34+","Pro-Myelocyte","HSC_-G-CSF","HSC_CD34+"),
    c("Macrophages","Monocytes","B cells","DC","Eosinophils","Neutrophils","T cells","ILC","NK cells","Basophils","Mast cells","Tgd","NKT","B cells, pro","Microglia")
  ),c("label.main","label.fine")])$label.fine
)]
cor.lin <- as.matrix(singler.lin@listData$scores)
order.lin <- c(
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Epithelial_cells","Keratinocytes","Neuroepithelial_cell"),
    c("Epithelial cells")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Smooth_muscle_cells","Fibroblasts","Chondrocytes","Osteoblasts","MSC","Tissue_stem_cells"),
    c("Stromal cells","Fibroblasts")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Endothelial_cells","Erythroblast","Platelets"),
    c("Endothelial cells")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("DC","B_cell","Neutrophil","T_cells","Monocyte","Macrophage","NK_cell","Neutrophils","CMP","GMP","MEP","Myelocyte","Pre-B_cell_CD34-","Pro-B_cell_CD34+","Pro-Myelocyte","HSC_-G-CSF","HSC_CD34+"),
    c("Macrophages","Monocytes","B cells","DC","Eosinophils","Neutrophils","T cells","ILC","NK cells","Basophils","Mast cells","Tgd","NKT","B cells, pro","Microglia")
  ),c("label.main","label.fine")])$label.fine)
)
cor.lin <- cor.lin[order.lin,]
cor.lin.ann <- columnAnnotation(Cell.Lineage=factor(sc10x$`Cell.Lineage`[match(singler.lin@rownames,sc10x$integrated_snn_res.5)][order.lin],levels=c("Epi","FMSt","Endo","Leu")),col=list(Cell.Lineage=c("Epi"=magma(4)[1],"FMSt"=magma(4)[2],"Endo"=magma(4)[3],"Leu"=magma(4)[4])))
postscript(paste0("./analysis/vis/diy/CorrCustom_lin.eps"))
Heatmap(scale(t(cor.lin)),name="Correlation Z-Score",cluster_rows=FALSE,cluster_columns=FALSE,col=rev(heat.colors(2)),top_annotation=cor.lin.ann,row_title="HPCA Cell Types",column_title_side="bottom",column_title="Human Prostate Clusters",show_row_names=FALSE)
dev.off()

Idents(sc10x.epi) <- "samples"
sc10x.epi <- subset(sc10x.epi,idents=c("D17PrPzF_Via","D27PrPzF_Via","D35PrPzF_Via","BPH327PrGF_Via","BPH340PrGF_Via","BPH342PrF_Via"))

sc10x.epi$Phenotype <- sc10x.epi$samples
Idents(sc10x.epi) <- "Phenotype"
sc10x.epi <- RenameIdents(sc10x.epi,"D17PrPzF_Via"="Normal Pz")
#sc10x.epi <- RenameIdents(sc10x.epi,"D17PrTzF_Via"="Normal")
sc10x.epi <- RenameIdents(sc10x.epi,"D27PrPzF_Via"="Normal Pz")
#sc10x.epi <- RenameIdents(sc10x.epi,"D27PrTzF_Via"="Normal")
sc10x.epi <- RenameIdents(sc10x.epi,"D35PrPzF_Via"="Normal Pz")
#sc10x.epi <- RenameIdents(sc10x.epi,"D35PrTzF_Via"="Normal")
sc10x.epi <- RenameIdents(sc10x.epi,"BPH327PrGF_Via"="BPH")
sc10x.epi <- RenameIdents(sc10x.epi,"BPH340PrGF_Via"="BPH")
sc10x.epi <- RenameIdents(sc10x.epi,"BPH342PrF_Via"="BPH")
sc10x.epi$Phenotype <- Idents(sc10x.epi)
sc10x.epi$Phenotype <- factor(sc10x.epi$Phenotype,levels=c("Normal Pz","BPH"))

sc10x.epi$`Cell.Type` <- sc10x.epi$pops
sc10x.epi$`Cell.Type` <- factor(sc10x.epi$`Cell.Type`,levels=c("BE","LE","Hillock","Club","NE"))
Idents(sc10x.epi) <- "Cell.Type"

postscript(paste0("./analysis/vis/diy/UMAP_epi.pops.eps"))
DimPlot(sc10x.epi)+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_epi.pops.split.eps"))
DimPlot(sc10x.epi,split.by="Phenotype")+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
Idents(sc10x.epi) <- "Phenotype"
sc10x.epi.normalpz <- subset(sc10x.epi,idents="Normal Pz")
sc10x.epi.bph <- subset(sc10x.epi,idents="BPH")
postscript(paste0("./analysis/vis/diy/UMAP_epi.pops.split_NormalPz.eps"))
DimPlot(sc10x.epi.normalpz,group.by="Cell.Type")+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_epi.pops.split_BPH.eps"))
DimPlot(sc10x.epi.bph,group.by="Cell.Type")+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_epi.pops.anchor.eps"))
DotPlot(sc10x.epi,features=rev(c("KRT5","KLK3","KRT13","SCGB3A1","CHGA")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

Idents(sc10x.fmst) <- "samples"
sc10x.fmst <- subset(sc10x.fmst,idents=c("D17PrPzF_Via","D27PrPzF_Via","D35PrPzF_Via","BPH327PrGF_Via","BPH340PrGF_Via","BPH342PrF_Via"))

sc10x.fmst$Phenotype <- sc10x.fmst$samples
Idents(sc10x.fmst) <- "Phenotype"
sc10x.fmst <- RenameIdents(sc10x.fmst,"D17PrPzF_Via"="Normal Pz")
#sc10x.fmst <- RenameIdents(sc10x.fmst,"D17PrTzF_Via"="Normal")
sc10x.fmst <- RenameIdents(sc10x.fmst,"D27PrPzF_Via"="Normal Pz")
#sc10x.fmst <- RenameIdents(sc10x.fmst,"D27PrTzF_Via"="Normal")
sc10x.fmst <- RenameIdents(sc10x.fmst,"D35PrPzF_Via"="Normal Pz")
#sc10x.fmst <- RenameIdents(sc10x.fmst,"D35PrTzF_Via"="Normal")
sc10x.fmst <- RenameIdents(sc10x.fmst,"BPH327PrGF_Via"="BPH")
sc10x.fmst <- RenameIdents(sc10x.fmst,"BPH340PrGF_Via"="BPH")
sc10x.fmst <- RenameIdents(sc10x.fmst,"BPH342PrF_Via"="BPH")
sc10x.fmst$Phenotype <- Idents(sc10x.fmst)
sc10x.fmst$Phenotype <- factor(sc10x.fmst$Phenotype,levels=c("Normal Pz","BPH"))

sc10x.fmst$`Cell.Type` <- sc10x.fmst$pops
sc10x.fmst$`Cell.Type` <- factor(sc10x.fmst$`Cell.Type`,levels=c("Fib","SM"))
Idents(sc10x.fmst) <- "Cell.Type"

postscript(paste0("./analysis/vis/diy/UMAP_fmst.pops.eps"))
DimPlot(sc10x.fmst)+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_fmst.pops.split.eps"))
DimPlot(sc10x.fmst,split.by="Phenotype")+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_fmst.pops.anchor.eps"))
DotPlot(sc10x.fmst,features=rev(c("DCN","MYL9")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

singler.epi@listData$scores <- singler.epi@listData$scores[,c(1,2,4,3,5)]
cor.epi <- as.matrix(singler.epi@listData$scores)[singler.epi@rownames %in% colnames(sc10x.epi),]
cor.epi <- cor.epi[order(sc10x.epi$`Cell.Type`[match(singler.epi@rownames[singler.epi@rownames %in% colnames(sc10x.epi)],names(sc10x.epi$`Cell.Type`))]),]
cor.epi.ann <- columnAnnotation(Cell.Type=factor(sc10x.epi$`Cell.Type`[match(singler.epi@rownames[singler.epi@rownames %in% colnames(sc10x.epi)],names(sc10x.epi$`Cell.Type`))][order(sc10x.epi$`Cell.Type`[match(singler.epi@rownames[singler.epi@rownames %in% colnames(sc10x.epi)],names(sc10x.epi$`Cell.Type`))])],levels=c("BE","LE","Hillock","Club","NE")),col=list(Cell.Type=c("BE"=brewer.pal(n=5,name="Dark2")[1],"LE"=brewer.pal(n=5,name="Dark2")[2],"Hillock"=brewer.pal(n=5,name="Dark2")[3],"Club"=brewer.pal(n=5,name="Dark2")[4],"NE"=brewer.pal(n=5,name="Dark2")[5])))
postscript(paste0("./analysis/vis/diy/CorrCustom_epi.eps"))
Heatmap(scale(t(cor.epi)),name="Correlation Z-Score",cluster_rows=FALSE,cluster_columns=FALSE,col=rev(heat.colors(2)),top_annotation=cor.epi.ann,row_title="Human Epi Cell Types",column_title_side="bottom",column_title="Human Epi Cells")
dev.off()

cor.fmst <- as.matrix(singler.fmst@listData$scores)[singler.fmst@rownames %in% colnames(sc10x.fmst),]
cor.fmst <- cor.fmst[order(sc10x.fmst$`Cell.Type`[match(singler.fmst@rownames[singler.fmst@rownames %in% colnames(sc10x.fmst)],names(sc10x.fmst$`Cell.Type`))]),]
cor.fmst.ann <- columnAnnotation(Cell.Type=factor(sc10x.fmst$`Cell.Type`[match(singler.fmst@rownames[singler.fmst@rownames %in% colnames(sc10x.fmst)],names(sc10x.fmst$`Cell.Type`))][order(sc10x.fmst$`Cell.Type`[match(singler.fmst@rownames[singler.fmst@rownames %in% colnames(sc10x.fmst)],names(sc10x.fmst$`Cell.Type`))])],levels=c("Fib","SM")),col=list(Cell.Type=c("Fib"=brewer.pal(n=3,name="Dark2")[1],"SM"=brewer.pal(n=3,name="Dark2")[2])))
postscript(paste0("./analysis/vis/diy/CorrCustom_fmst.eps"))
Heatmap(scale(t(cor.fmst)),name="Correlation Z-Score",cluster_rows=FALSE,cluster_columns=FALSE,col=rev(heat.colors(2)),top_annotation=cor.fmst.ann,row_title="Human FMSt Cell Types",column_title_side="bottom",column_title="Human FMSt Cells")
dev.off()

write.table(table(sc10x$`Cell.Type`,sc10x$samples),file="./analysis/vis/diy/Table_pops.Samples.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)
write.table(table(sc10x$`Cell.Type`,sc10x$Phenotype),file="./analysis/vis/diy/Table_pops.Phenotype.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)

Idents(sc10x) <- "lin"
data <- subset(sc10x,idents="Epi")
Idents(data) <- "Cell.Type"
data <- RenameIdents(data,"Hillock"="Urethra")
data <- RenameIdents(data,"Club"="Urethra")
deg <- FindAllMarkers(data,assay="SCT",slot="data",logfc.threshold=0,test.use="MAST",only.pos=TRUE)
write.table(deg,file=paste0("./analysis/vis/diy/DEG_pheno.Urethra.csv"),sep=",",quote=FALSE,row.names=FALSE,col.names=TRUE)

Idents(sc10x) <- "Cell.Type"
for (i in c("BE","LE","Hillock","Club","NE","Fib","SM")){
  data <- subset(sc10x,idents=i)
  Idents(data) <- "Phenotype"
  deg <- FindAllMarkers(data,assay="SCT",slot="data",logfc.threshold=0,test.use="MAST")
  write.table(deg,file=paste0("./analysis/vis/diy/DEG_pheno.",i,".csv"),sep=",",quote=FALSE,row.names=FALSE,col.names=TRUE)
}

postscript(paste0("./analysis/vis/diy/UMAP_epi.CHGA.eps"))
FeaturePlot(sc10x.epi,features="CHGA",cols=c("lightgrey","darkred"),pt.size=2.5)+theme_cowplot()
dev.off()
