library(Seurat)
library(SingleR)
library(ggplot2)
library(cowplot)
library(gridExtra)
library(RColorBrewer)
library(viridis)
library(scales)
library(ComplexHeatmap)
library(autothresholdr)
library(dplyr)
library(qusage)
options(bitmapType="cairo")

setwd("../")

source("./r.scripts/sc-TissueMapper_functions.R")

#load("./genesets/sc10x.epi.id.rda")
#deg.ne <- FindMarkers(sc10x.epi,ident.1="NE",assay="SCT",slot="data",logfc.threshold=log(1.5,2),test.use="MAST",only.pos=TRUE)
#deg.ne <- as.character(rownames(deg.ne))
#convert <- read.delim("./genesets/Ensemble.mus-hum.txt")
#deg.ne <- deg.ne[deg.ne %in% convert$Human.gene.name]
#deg.ne <- as.character(convert$Gene.name[match(deg.ne,convert$Human.gene.name)])
#anchor <- c("Chga","Chgb","Calca")

load("./analysis/sc10x.id.rda")
load("./analysis/singler_objects.RData")

if (!dir.exists(paste0("./analysis/vis/diy"))){
  dir.create(paste0("./analysis/vis/diy"))
}


sc10x$Region <- sc10x$samples
Idents(sc10x) <- "Region"
sc10x <- RenameIdents(sc10x,"musAd001_PrF"="Prostate")
sc10x <- RenameIdents(sc10x,"musAd002_PrF"="Prostate")
sc10x <- RenameIdents(sc10x,"musAd003_PrF_St"="Prostate")
sc10x <- RenameIdents(sc10x,"musAd007_PrFcol_GEX"="Prostate")
sc10x <- RenameIdents(sc10x,"musAd002_UrF"="Urethra")
sc10x <- RenameIdents(sc10x,"musAd004n5_UrF_GEX"="Urethra")
sc10x$Region <- Idents(sc10x)
sc10x$Region <- factor(sc10x$Region,levels=c("Prostate","Urethra"))

sc10x$`Cell.Lineage` <- sc10x$lin
Idents(sc10x) <- "Cell.Lineage"
sc10x$`Cell.Lineage` <- Idents(sc10x)
sc10x$`Cell.Lineage` <- factor(sc10x$`Cell.Lineage`,levels=c("Epi","FMSt","Endo","Leu"))
Idents(sc10x) <- "Cell.Lineage"

postscript(paste0("./analysis/vis/diy/UMAP_lin.eps"))
DimPlot(sc10x)+scale_color_viridis(discrete=TRUE,option="magma")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_lin.split.eps"))
DimPlot(sc10x,split.by="Region")+scale_color_viridis(discrete=TRUE,option="magma")+theme_cowplot()
dev.off()

lin.se <- ImmGenData()
ref.lin <- c(
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Epithelial_cells","Keratinocytes","Neuroepithelial_cell"),
    c("Epithelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Smooth_muscle_cells","Fibroblasts","Chondrocytes","Osteoblasts","MSC","Tissue_stem_cells"),
    c("Stromal cells","Fibroblasts")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Endothelial_cells","Erythroblast","Platelets"),
    c("Endothelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("DC","B_cell","Neutrophil","T_cells","Monocyte","Macrophage","NK_cell","Neutrophils","CMP","GMP","MEP","Myelocyte","Pre-B_cell_CD34-","Pro-B_cell_CD34+","Pro-Myelocyte","HSC_-G-CSF","HSC_CD34+"),
    c("Macrophages","Monocytes","B cells","DC","Eosinophils","Neutrophils","T cells","ILC","NK cells","Basophils","Mast cells","Tgd","NKT","B cells, pro","Microglia")
  ),c("label.main","label.fine")])$label.fine
)[c(
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Epithelial_cells","Keratinocytes","Neuroepithelial_cell"),
    c("Epithelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Smooth_muscle_cells","Fibroblasts","Chondrocytes","Osteoblasts","MSC","Tissue_stem_cells"),
    c("Stromal cells","Fibroblasts")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Endothelial_cells","Erythroblast","Platelets"),
    c("Endothelial cells")
  ),c("label.main","label.fine")])$label.fine,
  unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("DC","B_cell","Neutrophil","T_cells","Monocyte","Macrophage","NK_cell","Neutrophils","CMP","GMP","MEP","Myelocyte","Pre-B_cell_CD34-","Pro-B_cell_CD34+","Pro-Myelocyte","HSC_-G-CSF","HSC_CD34+"),
    c("Macrophages","Monocytes","B cells","DC","Eosinophils","Neutrophils","T cells","ILC","NK cells","Basophils","Mast cells","Tgd","NKT","B cells, pro","Microglia")
  ),c("label.main","label.fine")])$label.fine
) %in% colnames(singler.lin@listData$scores)]
singler.lin@listData$scores <- singler.lin@listData$scores[,ref.lin]
cor.lin <- as.matrix(singler.lin@listData$scores)
order.lin <- c(
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Epithelial_cells","Keratinocytes","Neuroepithelial_cell"),
    c("Epithelial cells")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Smooth_muscle_cells","Fibroblasts","Chondrocytes","Osteoblasts","MSC","Tissue_stem_cells"),
    c("Stromal cells","Fibroblasts")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("Endothelial_cells","Erythroblast","Platelets"),
    c("Endothelial cells")
  ),c("label.main","label.fine")])$label.fine),
  which(singler.lin$labels %in% unique(lin.se@colData[lin.se@colData[,"label.main"] %in% c(
    c("DC","B_cell","Neutrophil","T_cells","Monocyte","Macrophage","NK_cell","Neutrophils","CMP","GMP","MEP","Myelocyte","Pre-B_cell_CD34-","Pro-B_cell_CD34+","Pro-Myelocyte","HSC_-G-CSF","HSC_CD34+"),
    c("Macrophages","Monocytes","B cells","DC","Eosinophils","Neutrophils","T cells","ILC","NK cells","Basophils","Mast cells","Tgd","NKT","B cells, pro","Microglia")
  ),c("label.main","label.fine")])$label.fine)
)
cor.lin <- cor.lin[order.lin,]
cor.lin.ann <- columnAnnotation(Cell.Lineage=factor(sc10x$`Cell.Lineage`[match(singler.lin@rownames,sc10x$integrated_snn_res.1)][order.lin],levels=c("Epi","FMSt","Endo","Leu")),col=list(Cell.Lineage=c("Epi"=magma(4)[1],"FMSt"=magma(4)[2],"Endo"=magma(4)[3],"Leu"=magma(4)[4])))
postscript(paste0("./analysis/vis/diy/CorrCustom_lin.eps"))
Heatmap(scale(t(cor.lin)),name="Correlation Z-Score",cluster_rows=FALSE,cluster_columns=FALSE,col=rev(heat.colors(2)),top_annotation=cor.lin.ann,row_title="ImmGen Cell Types",column_title_side="bottom",column_title="Mouse Clusters",show_row_names=FALSE)
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_lin.anchor.eps"))
DotPlot(sc10x,features=rev(c("Epcam","Cdh1","Dcn","Myl9","Pecam1","Vwf","Ptprc")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

sc10x.epi$Region <- sc10x.epi$samples
Idents(sc10x.epi) <- "Region"
sc10x.epi <- RenameIdents(sc10x.epi,"musAd001_PrF"="Prostate")
sc10x.epi <- RenameIdents(sc10x.epi,"musAd002_PrF"="Prostate")
sc10x.epi <- RenameIdents(sc10x.epi,"musAd003_PrF_St"="Prostate")
sc10x.epi <- RenameIdents(sc10x.epi,"musAd007_PrFcol_GEX"="Prostate")
sc10x.epi <- RenameIdents(sc10x.epi,"musAd002_UrF"="Urethra")
sc10x.epi <- RenameIdents(sc10x.epi,"musAd004n5_UrF_GEX"="Urethra")
sc10x.epi$Region <- Idents(sc10x.epi)
sc10x.epi$Region <- factor(sc10x.epi$Region,levels=c("Prostate","Urethra"))


Idents(sc10x.epi) <- "integrated_snn_res.0.1"
cells.ap <- WhichCells(sc10x.epi,idents="0")
cells.vp <- WhichCells(sc10x.epi,idents="3")
cells.ed <- WhichCells(sc10x.epi,idents="4")
cells.dlp <- WhichCells(sc10x.epi,idents="5")

sc10x.epi$`Cell.Type` <- sc10x.epi$hu_pops
Idents(sc10x.epi) <- "Cell.Type"
sc10x.epi <- SetIdent(sc10x.epi,cell=cells.ap,value="APr LE")
sc10x.epi <- SetIdent(sc10x.epi,cell=cells.vp,value="VPr LE")
sc10x.epi <- SetIdent(sc10x.epi,cell=cells.ed,value="ED")
sc10x.epi <- SetIdent(sc10x.epi,cell=cells.dlp,value="DLPr LE")
sc10x.epi$`Cell.Type` <- Idents(sc10x.epi)
sc10x.epi$`Cell.Type` <- factor(sc10x.epi$`Cell.Type`,levels=c("BE","Ur","VPr LE","DLPr LE","APr LE","ED"))
Idents(sc10x.epi) <- "Cell.Type"

postscript(paste0("./analysis/vis/diy/UMAP_epi.eps"))
DimPlot(sc10x.epi)+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_epi.split.eps"))
DimPlot(sc10x.epi,split.by="Region")+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()

Idents(sc10x.epi) <- "Region"
sc10x.epi.pr <- subset(sc10x.epi,idents="Prostate")
sc10x.epi.ur <- subset(sc10x.epi,idents="Urethra")
Idents(sc10x.epi) <- "Cell.Type"
Idents(sc10x.epi.pr) <- "Cell.Type"
Idents(sc10x.epi.ur) <- "Cell.Type"
postscript(paste0("./analysis/vis/diy/UMAP_epi.split.pr.eps"))
DimPlot(sc10x.epi.pr)+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_epi.split.ur.eps"))
DimPlot(sc10x.epi.ur)+scale_color_brewer(palette="Dark2")+theme_cowplot()
dev.off()

singler.epi@listData$scores <- singler.epi@listData$scores[,c(1,3,2)]
cor.epi <- as.matrix(singler.epi@listData$scores)
cor.epi <- cor.epi[c(2,3,1,4,6,5),]
cor.epi.ann <- columnAnnotation(Cell.Type=factor(c("BE","Ur","VP LE","DLP LE","AP LE","ED"),levels=c("BE","Ur","VP LE","DLP LE","AP LE","ED")),col=list(ID=c("BE"="brown3","Ur"="brown2","LE"="brown1"),Cell.Type=c("BE"=brewer.pal(n=6,name="Dark2")[1],"Ur"=brewer.pal(n=6,name="Dark2")[2],"VP LE"=brewer.pal(n=6,name="Dark2")[3],"DLP LE"=brewer.pal(n=6,name="Dark2")[4],"AP LE"=brewer.pal(n=6,name="Dark2")[5],"ED"=brewer.pal(n=6,name="Dark2")[6])))
postscript(paste0("./analysis/vis/diy/CorrCustom_epi.eps"))
Heatmap(scale(t(cor.epi)),name="Correlation Z-Score",cluster_rows=FALSE,cluster_columns=FALSE,col=rev(heat.colors(2)),top_annotation=cor.epi.ann,row_title="Human Cell Types",column_title_side="bottom",column_title="Mouse Epi Clusters")
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_epi.anchor.eps"))
DotPlot(sc10x.epi,features=rev(c("Krt5","Krt4","Sbp","Msmb","Tgm4","Svs2")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

postscript(paste0("./analysis/vis/diy/Vln_epi.anchor.eps"))
VlnPlot(sc10x.epi,features=c("Krt5","Krt4","Sbp","Msmb","Tgm4","Svs2"),pt.size=0.1,cols=brewer.pal(n=6,name="Dark2"))+theme_cowplot()
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_epi.sc.eps"))
DotPlot(sc10x.epi,features=rev(c("Svs2","Krt8","Cd24a","Dpp4","Pbsn","Msmb","Sbp","Tgm4","Trp63","Krt5","Krt19","Krt4","Ly6d","Psca","Tacstd2","Ly6e","Ly6a","Prom1")),dot.scale=10,cols=rev(heat.colors(2)))+theme(axis.text.x=element_text(angle=45,hjust=1))
dev.off()

postscript(paste0("./analysis/vis/diy/Dot_epi.sc_SUB.eps"))
DotPlot(sc10x.epi,features=rev(c("Krt4","Tacstd2","Nkx3-1","Dpp4")),dot.scale=10,cols=rev(heat.colors(2)))+theme(axis.text.x=element_text(angle=45,hjust=1))
dev.off()

sc10x.epi$`Cell.ID` <- factor(sc10x.epi$hu_pops,levels=c("BE","Ur","LE"))
Idents(sc10x.epi) <- "Cell.ID"
postscript(paste0("./analysis/vis/diy/UMAP_epi_hupop.eps"))
DimPlot(sc10x.epi)+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/UMAP_epi_hupop.split.eps"))
DimPlot(sc10x.epi,split.by="Region")+theme_cowplot()
dev.off()
postscript(paste0("./analysis/vis/diy/Dot_epi_hupop.anchor.eps"))
DotPlot(sc10x.epi,features=rev(c("Krt5","Trp63","Psca","Krt4","Pbsn","Msmb")),dot.scale=10,cols=rev(heat.colors(2)))
dev.off()

deg.lin <- FindAllMarkers(sc10x,assay="SCT",slot="data",logfc.threshold=0,test.use="MAST",only.pos=TRUE)
deg.epi <- FindAllMarkers(sc10x.epi,assay="SCT",slot="data",logfc.threshold=0,test.use="MAST",only.pos=TRUE)

write.table(deg.lin,file="./analysis/vis/diy/DEG_lin.csv",sep=",",quote=FALSE,row.names=FALSE,col.names=TRUE)
write.table(deg.epi,file="./analysis/vis/diy/DEG_epi.csv",sep=",",quote=FALSE,row.names=FALSE,col.names=TRUE)

Idents(sc10x) <- "integrated_snn_res.1"
postscript(paste0("./analysis/vis/diy/UMAP_lin.res1.eps"))
DimPlot(sc10x)+theme_cowplot()
dev.off()
Idents(sc10x.epi) <- "integrated_snn_res.0.1"
postscript(paste0("./analysis/vis/diy/UMAP_epi.res0.1.eps"))
DimPlot(sc10x.epi)+theme_cowplot()
dev.off()

write.table(table(sc10x$`Cell.Lineage`,sc10x$samples),file="./analysis/vis/diy/Table_Lin.Samples.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)
write.table(table(sc10x$`Cell.Lineage`,sc10x$Region),file="./analysis/vis/diy/Table_Lin.Region.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)

write.table(table(sc10x.epi$`Cell.Type`,sc10x.epi$samples),file="./analysis/vis/diy/Table_Epi.Samples.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)
write.table(table(sc10x.epi$`Cell.Type`,sc10x.epi$Region),file="./analysis/vis/diy/Table_Epi.Region.csv",sep=",",quote=FALSE,row.names=TRUE,col.names=NA)

#deg.ne <- rownames(sc10x.epi)[rownames(sc10x.epi) %in% deg.ne]
#results <- scScore(sc10x.epi,score="NE",geneset=as.list(deg.ne),cut.pt=0.95,anchor=anchor)
#table(results[[1]]$NE)

E.MTAB.4991_BE <- read.delim("./genesets/BEvsBE.rest.txt", header=TRUE, comment.char="#", stringsAsFactors=FALSE)
E.MTAB.4991_LE <- read.delim("./genesets/LEvsLE.rest.txt", header=TRUE, comment.char="#", stringsAsFactors=FALSE)
E.MTAB.4991_OE <- read.delim("./genesets/OEvsOE.rest.txt", header=TRUE, comment.char="#", stringsAsFactors=FALSE)

E.MTAB.4991_BE <- unique(E.MTAB.4991_BE[E.MTAB.4991_BE$Fold.Change > 0 & E.MTAB.4991_BE$FDR.P.val <= 0.05 & (E.MTAB.4991_BE$Group == "Coding" | E.MTAB.4991_BE$Group == "Multiple_Complex") & E.MTAB.4991_BE$Gene.Symbol != "","Gene.Symbol"])
E.MTAB.4991_BE <- list(E.MTAB.4991_BE)
names(E.MTAB.4991_BE) <- "Basal"
E.MTAB.4991_LE <- unique(E.MTAB.4991_LE[E.MTAB.4991_LE$Fold.Change > 0 & E.MTAB.4991_LE$FDR.P.val <= 0.05 & (E.MTAB.4991_LE$Group == "Coding" | E.MTAB.4991_LE$Group == "Multiple_Complex") & E.MTAB.4991_LE$Gene.Symbol != "","Gene.Symbol"])
E.MTAB.4991_LE <- list(E.MTAB.4991_LE)
names(E.MTAB.4991_LE) <- "Luminal"
E.MTAB.4991_OE <- unique(E.MTAB.4991_OE[E.MTAB.4991_OE$Fold.Change > 0 & E.MTAB.4991_OE$FDR.P.val <= 0.05 & (E.MTAB.4991_OE$Group == "Coding" | E.MTAB.4991_OE$Group == "Multiple_Complex") & E.MTAB.4991_OE$Gene.Symbol != "","Gene.Symbol"])
E.MTAB.4991_OE <- list(E.MTAB.4991_OE)
names(E.MTAB.4991_OE) <- "LSCmed"

E.MTAB.4991 <- list()
E.MTAB.4991 <- c(E.MTAB.4991_BE,E.MTAB.4991_LE,E.MTAB.4991_OE)

sc10x.epi$CellType <- sc10x.epi$`Cell.Type`
Idents(sc10x.epi) <- "CellType"
sc10x.epi <- RenameIdents(sc10x.epi,"VPr LE"="VPrLE")
sc10x.epi <- RenameIdents(sc10x.epi,"DLPr LE"="DLPrLE")
sc10x.epi <- RenameIdents(sc10x.epi,"APr LE"="APrLE")
sc10x.epi$CellType <- Idents(sc10x.epi)
sc10x.epi$CellType <- factor(sc10x.epi$CellType,levels=c("BE","Ur","VPrLE","DLPrLE","APrLE","ED"))
results <- scQuSAGE(sc10x.epi,gs=E.MTAB.4991,type="sm",id="CellType",nm="E.MTAB.4991",print="umap")

postscript(paste0("./analysis/vis/diy/UMAP_epi.Calca.eps"))
FeaturePlot(sc10x.epi,features="Calca",cols=c("lightgrey","darkred"),pt.size=2.5)+theme_cowplot()
dev.off()

Idents(sc10x.epi) <- "Cell.Type"
deg.epi <- FindAllMarkers(sc10x.epi,assay="SCT",slot="data",logfc.threshold=0,test.use="MAST",only.pos=TRUE)
write.table(deg.epi,file="./analysis/vis/diy/DEG_epi.CellType.csv",sep=",",quote=FALSE,row.names=FALSE,col.names=TRUE)
