gc()
library(methods)
library(optparse)
library(Seurat)
library(readr)
library(readxl)
library(fBasics)
library(pastecs)
library(qusage)
library(RColorBrewer)
library(monocle)
library(dplyr)
library(viridis)
library(gridExtra)
library(SingleR)
library(sctransform)
library(autothresholdr)
library(ggplot2)

options(bitmapType="cairo")
options(future.globals.maxSize= 1000000000000)

option_list=list(
  make_option("--p",action="store",default="muBl",type='character',help="Project Name"),
  make_option("--s",action="store",default="mu",type='character',help="Species")
)
opt=parse_args(OptionParser(option_list=option_list))
rm(option_list)


setwd("../")

source("./r.scripts/sc-TissueMapper_functions.R")
source("./r.scripts/sc-TissueMapper_process.R")

project.name=opt$p

scFolders()

results <- scLoad(p=project.name,cellranger=3,aggr=FALSE,ncell=0,nfeat=0)
sc10x <- results[[1]]
sc10x.groups <- results[[2]]
rm(results)

sc10x <- sc10x[as.character(unlist(sc10x.groups[sc10x.groups$ALL==1,1]))]

results <- scQC(sc10x,sp=opt$s,feature=c("nFeature_RNA"))
sc10x <- results[[1]]
counts.cell.raw <- results[[2]]
counts.gene.raw <- results[[3]]
counts.cell.filtered.1umi <- results[[4]]
counts.gene.filtered.1umi <- results[[5]]
rm(results)

results <- scQC(sc10x,sp=opt$s,feature="percent.mito")
sc10x <- results[[1]]
counts.cell.filtered.2mito <- results[[4]]
counts.gene.filtered.2mito <- results[[5]]
rm(results)

results <- scQC(sc10x,sp=opt$s,feature="nCount_RNA")
sc10x <- results[[1]]
counts.cell.filtered.3gene <- results[[4]]
counts.gene.filtered.3gene <- results[[5]]
rm(results)

#results <- scQC(sc10x,sp=opt$s,feature="percent.ribo")
#sc10x <- results[[1]]
#counts.cell.filtered.4ribo <- results[[4]]
#counts.gene.filtered.4ribo <- results[[5]]
#rm(results)

counts.cell.filtered <- counts.cell.filtered.3gene
counts.gene.filtered <- counts.gene.filtered.3gene

# results <- scCellCycle(sc10x,sub=FALSE,sp="hu")
# sc10x <- results[[1]]
# genes.s <- results[[2]]
# genes.g2m <- results[[3]]
# rm(results)

sc10x <- sc10x[as.character(unlist(sc10x.groups[sc10x.groups$Deep==1,1]))]

merges <- NULL
for (i in names(counts.cell.filtered[as.character(unlist(sc10x.groups[sc10x.groups$Deep==1,1]))])){
  if (counts.cell.filtered[[i]]<750){
    merges <- c(merges,i)
  }
}
rm(i)
if (length(merges)>1){
  sc10x.merge <- merge(x=sc10x[[merges[1]]],y=sc10x[merges[-1]])
  sc10x[merges] <- NULL
  sc10x$merge <- sc10x.merge
  rm(sc10x.merge)
} else if (length(merges)==1){
  merges <- c(merges,names(counts.cell.filtered[counts.cell.filtered == min(sapply(counts.cell.filtered[setdiff(as.character(unlist(sc10x.groups[sc10x.groups$Deep==1,1])),merges)],min))]))
  sc10x.merge <- merge(x=sc10x[[merges[1]]],y=sc10x[merges[-1]])
  sc10x[merges] <- NULL
  sc10x$merge <- sc10x.merge
}
rm(merges)

if (length(sc10x)>1){
  sc10x <- scCCA(sc10x)
  sc10x@project.name <- project.name
  sc10x$ALL <- "ALL"
}

results <- scPC(sc10x,pc=100,hpc=0.9,file="pre.stress",print="2")
sc10x <- results[[1]]
pc.use.prestress <- results[[2]]
rm(results)

sc10x <- scCluster(sc10x,res=0.5,red="pca",dim=pc.use.prestress,print="2",folder="pre.stress")

if (opt$s == "hu"){
  genes.stress <- read_delim("./genesets/genes.deg.Stress.csv",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  genes.stress <- genes.stress[1]
  colnames(genes.stress) <- "scDWS.Stress"
  anchor <- c("EGR1","FOS","JUN")
} else if (opt$s == "mu"){
  genes.stress <- read_delim("./genesets/van.den.Brink1.txt",",",escape_double=FALSE,trim_ws=TRUE,col_names=TRUE)
  genes.stress <- genes.stress[1]
  colnames(genes.stress) <- "van.den.Brink1.Stress"
  anchor <- c("Egr1","Fos","Jun")
}

results <- scScore(sc10x,score="Stress",geneset=as.list(genes.stress),cut.pt="renyi",anchor=anchor)
sc10x.preStress <- results[[1]]
sc10x <- results[[2]]
rm(results)
counts.cell.destress <- as.list(table(sc10x$samples))
rm(anchor)

#if (opt$p == "muPr" || opt$p == "muPrUr"){
#  hto <- as.character(unlist(sc10x.groups[sc10x.groups$HTO==1,1]))
#  sc10x.hto.data <- Read10X(data.dir=paste0("./analysis/DATA/10x/",hto,"/filtered_feature_bc_matrix/"))
#  sc10x.hto <- CreateSeuratObject(counts=sc10x.hto.data$`Gene Expression`,project=hto)
#  sc10x.hto[["HTO"]] <- CreateAssayObject(counts=sc10x.hto.data$`Antibody Capture`)
#  rm(sc10x.hto.data)
#  sc10x.hto <- NormalizeData(object=sc10x.hto,assay="HTO",normalization.method="CLR")
#  sc10x.hto <- HTODemux(sc10x.hto,assay="HTO",positive.quantile=.99)
#  sc10x.hto <- subset(sc10x.hto,cells=substr(names(sc10x$samples)[sc10x$samples == paste0(hto,"_GEX")],start=1,stop=16))
#  # HTOHeatmap(sc10x.hto)
#  # plot1 <- FeatureScatter(sc10x.hto,feature1 = "hto_Anterior-Prostate",feature2 = "hto_Ventral-Prostate",group.by = "HTO_maxID")
#  # plot2 <- FeatureScatter(sc10x.hto,feature1 = "hto_Anterior-Prostate",feature2 = "hto_Dorsolateral-Prostate",group.by = "HTO_maxID")
#  # plot3 <- FeatureScatter(sc10x.hto,feature1 = "hto_Dorsolateral-Prostate",feature2 = "hto_Ventral-Prostate",group.by = "HTO_maxID")
#  # grid.arrange(plot1,plot2,plot3,ncol=1)
#  Idents(sc10x) <- "samples"
#  Idents(sc10x,cells=paste0(names(sc10x.hto$HTO_maxID)[sc10x.hto$HTO_maxID=="Anterior-Prostate"],"_4")) <- paste0(hto,"_AP")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$HTO_maxID)[sc10x.hto$HTO_maxID=="Dorsolateral-Prostate"],"_4")) <- paste0(hto,"_DLP")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$HTO_maxID)[sc10x.hto$HTO_maxID=="Ventral-Prostate"],"_4")) <- paste0(hto,"_VP")
#  sc10x$HTO_maxID <- Idents(sc10x)
#  sc10x$samples_HTO <- Idents(sc10x)
#  Idents(sc10x,cells=paste0(names(sc10x.hto$hash.ID)[sc10x.hto$hash.ID=="Anterior-Prostate"],"_4")) <- paste0(hto,"_AP")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$hash.ID)[sc10x.hto$hash.ID=="Dorsolateral-Prostate"],"_4")) <- paste0(hto,"_DLP")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$hash.ID)[sc10x.hto$hash.ID=="Ventral-Prostate"],"_4")) <- paste0(hto,"_VP")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$hash.ID)[sc10x.hto$hash.ID=="Negative"],"_4")) <- paste0(hto,"_negative")
#  Idents(sc10x,cells=paste0(names(sc10x.hto$hash.ID)[sc10x.hto$hash.ID=="Doublet"],"_4")) <- paste0(hto,"_doublet")
#  sc10x$hash.ID <- Idents(sc10x)
#}

results <- scPC(sc10x,pc=100,hpc=0.9,file="post.stress",print="2")
sc10x <- results[[1]]
pc.use.poststress <- results[[2]]
rm(results)

res <- c(seq(0.1,0.5,0.1),0.75,seq(1,5,1))
sc10x <- scCluster(sc10x,res=res,red="pca",dim=pc.use.poststress,print="2",folder="ALL")

#if (opt$p == "muPr" || opt$p == "muPrUr"){
#  rm(sc10x.hto.data)
#  rm(sc10x.hto)
#  Idents(sc10x) <- "samples"
#  sc10x.tmp <- subset(sc10x,ident=paste0(hto,"_GEX"))
#  postscript(paste0("./analysis/vis/ALL/HTO_HTO_maxID.eps"))
#  DimPlot(sc10x.tmp,reduction="umap",group.by="HTO_maxID")
#  dev.off()
#  postscript(paste0("./analysis/vis/ALL/HTO_hash.ID.eps"))
#  DimPlot(sc10x.tmp,reduction="umap",group.by="hash.ID")
#  dev.off()
#}

DefaultAssay(object=sc10x) <- "SCT"

#scShinyOutput(sc10x,anal="raw")

save(sc10x,file=paste0("./analysis/sc10x.raw.rda"))
save.image(file="./analysis/sc10x.raw.RData")
